#!/bin/bash
#
# This script acts as a wrapper around the server side extensions for Swarm. Whilst the
# configure-swarm.sh script will install the extensions first time, there isn't much
# control over what is done.
#
# The purpose of this script is to provide some user control, hopefully without having
# to know too much about the underlying commands.
#

set -e
shopt -s nullglob

USER=""
PORT=""

function usage() {
    echo "$(basename "$0") [-y] [-u <p4_super_user>] [-p <p4port>] <command>"
    echo "You need to specify one of the following commands:"
    echo "  list    - List the currently installed extensions on this P4D"
    echo "  install - Install the swarm extension, using saved config"
    echo "            Optionally takes filename of extension to install"
    echo "  delete  - Remove the swarm extension"
    echo "  config  - Open the configuration for the extensions"
    echo "  tokens  - List the 'trigger' tokens for this swarm server"
    echo "  enable  - Enable currently installed swarm extensions"
    echo "  disable - Disable currently installed swarm extensions"
    echo "  save    - Save the current config to files in /opt/perforce/etc"
    echo "  load    - Load the config from the files in /opt/perforce/etc"
    echo "  ping    - Run the extension test to make sure they are working"
}

function die() {
    echo $*
    exit 1
}

function yn() {
    [ ! -z $YES ] && return
    echo "Are you really sure (y/n)?"
    read -r yn
    if [ "$yn" != "y" ]
    then
      exit 0
    fi
}

while getopts "u:p:y" opt
do
    case $opt in
        u)
            USER=$OPTARG
            ;;
        p)
            PORT=$OPTARG
            ;;
        y)
            YES=1
            ;;
        *)
            usage
            exit 1
            ;;
    esac
done
shift $((OPTIND - 1))
command=$1
optional=$2

if [ -z "$command" ]
then
    usage
    exit 1
fi

P4="p4"
if [ ! -z "$PORT" ]
then
    P4="$P4 -p $PORT"
fi
if [ ! -z "$USER" ]
then
    P4="$P4 -u $USER"
fi


echo -n "Checking to see if you are a super user... "

$P4 protects -m

level=$($P4 protects -m)
if [ "$level" != "super" ]
then
    echo "You are not a super user. Use -u <user> to specify a super user."
    exit 1
fi

PACKAGEFILE=/opt/perforce/swarm/p4-bin/extensions/helix-swarm.p4-extension
NAME=Perforce::helix-swarm
INSTANCE=swarm
CONFIG=/opt/perforce/etc/swarm-extension.conf

case $command in
    "help")
        usage
        ;;
    "list")
        # List all installed extensions and their configurations.
        # Not just Swarm extensions are listed.
        echo -e "\nGlobal Configs\n"
        $P4 extension --list --type extensions #| grep -e "... extension" -e "... rev"
        echo -e "\nInstance Configs\n"
        $P4 extension --list --type configs #| grep -e "... config" -e "... revision" -e "... type"
        ;;
    "delete")
        # Delete the Swarm extension
        echo "Deleting the Swarm extension from server" 
        yn
        $P4 extension --delete $NAME -y || die "Failed to delete extension"
        ;;
    "install")
        # Install the Swarm extension. If there is a helix-swarm.package file in the
        # current directory, then this is used and sign checking is disabled. Otherwise
        # installs the one from the usual location.
        #
        # After installation, if there are saved configurations (see 'save'), then these
        # are automatically applied.
        UNSIGNED=""
        if [ ! -z "$optional" ]
        then
            if [ ! -r "$optional" ]
            then
                echo "Unable to read file '$optional'"
                exit 2
            fi
            PACKAGEFILE=$optional
            UNSIGNED="--allow-unsigned"
        fi
        echo "Installing the extension '$PACKAGEFILE'"
        yn
        if [ -r "$PACKAGEFILE" ]
        then
            $P4 extension $UNSIGNED --install "$PACKAGEFILE" -y || die "Failed to install extension"
        else
            echo "Unable to find '$PACKAGEFILE' to install"
            exit 2
        fi
        if [ -f "$CONFIG" ]
        then
            echo "Reading configuration from $CONFIG"
            $P4 extension --configure $NAME -i < "$CONFIG"
            if [ -f "$CONFIG.inst" ]
            then
                echo "Reading configuration from $CONFIG.inst"
                $P4 extension --configure $NAME --name $INSTANCE -i < "$CONFIG.inst"
            else
                $P4 extension --configure $NAME --name $INSTANCE -o | $P4 extension --configure $NAME --name $INSTANCE -i
            fi
        else
            $P4 extension --configure $NAME
            $P4 extension --configure $NAME --name $INSTANCE -o | $P4 extension --configure $NAME --name $INSTANCE -i
        fi
        ;;
    "config")
        # Run a manual configuration, once for the global config, then for the instance
        # configuration. By default, will open things up in vim to be edited.
        $P4 extension --configure $NAME
        $P4 extension --configure $NAME --name $INSTANCE
        ;;
    "tokens")
        # List the trigger tokens in the Swarm 'tokens' directory. Will look in the top
        # level directory, and also in any server instances if Multi-P4D is configured.
        DATADIR="/opt/perforce/swarm/data"
        if [ -d "$DATADIR/queue/tokens" ]
        then
            echo "Available tokens:"
            ls $DATADIR/queue/tokens
        else
            ls $DATADIR/queue/tokens 2>&1 | grep -q denied && echo "You do not have permission to read $DATADIR/queue/tokens"
        fi
        if [ -d "$DATADIR/servers" ]
        then
            for i in $DATADIR/servers/*
            do
                if [ -d "$i/queue/tokens" ]
                then
                    echo "Tokens in instance $i"
                    ls "$i/queue/tokens"
                fi
            done
        fi
        ;;
    "disable")
        # Temporarily disables the Swarm extension without ininstalling it.
        $P4 --field "ExtEnabled=false" extension --configure $NAME -o | $P4 extension --configure $NAME -i >/dev/null && echo "Disabled"
        ;;
    "enable")
        # Enables the Swarm extension.
        $P4 --field "ExtEnabled=true" extension --configure $NAME -o | $P4 extension --configure $NAME -i >/dev/null && echo "Enabled"
        ;;
    "save")
        # Dumps the existing configuration to files in /opt/perforce/etc
        if [ ! -d /opt/perforce/etc ]
        then
            echo "There is no output directory at /opt/perforce/etc"
            exit 1
        fi
        if [ ! -w /opt/perforce/etc ]
        then
            echo "Do not have permission to write to /opt/perforce/etc"
            exit 1
        fi
        $P4 extension --configure $NAME -o > $CONFIG && echo "Written to $CONFIG"
        $P4 extension --configure $NAME --name swarm -o > $CONFIG.inst && echo "Written to $CONFIG.inst"
        ;;
    "load")
        # If there are configuration files in /opt/perforce/etc then they are loaded
        if [ -f $CONFIG ]
        then
          $P4 extension --configure $NAME -i < $CONFIG > /dev/null && echo "Loaded $CONFIG onto server"
        else
          echo "No configuration file found at $CONFIG"
          exit 2
        fi
        if [ -f $CONFIG.inst ]
        then
          $P4 extension --configure $NAME --name swarm -i < $CONFIG.inst > /dev/null && echo "Loaded $CONFIG.inst onto server"
        else
          echo "No instance configuration file found at $CONFIG.inst"
          exit 2
        fi
        ;;
    "ping")
        # Runs swarm || [ "$command" == "help" ] ping and swarm version to check the status of the extension.
        $P4 extension --run swarm ping
        $P4 extension --run swarm version
        ;;
    *)
        echo "Unknown command '$command'."
        usage
        exit 1
        ;;
esac
